@@ -61,7 +61,7 @@ or the C<spamd>/C<spamc> tools provided.
 package Mail::SpamAssassin;
 use strict;
 use warnings;
-use bytes;
+# use bytes;
 use re 'taint';
 
 require 5.006_001;
@@ -87,37 +87,32 @@ use Time::HiRes qw(time);
 use Cwd;
 use Config;
 
-use vars qw{
-  @ISA $VERSION $SUB_VERSION @EXTRA_VERSION $IS_DEVEL_BUILD $HOME_URL
-  @default_rules_path @default_prefs_path
-  @default_userprefs_path @default_userstate_dir
-  @site_rules_path
-};
+our $VERSION = "3.004003";      # update after release (same format as perl $])
+#our $IS_DEVEL_BUILD = 1;        # 1 for devel build 
+our $IS_DEVEL_BUILD = 0;        # 0 for release versions including rc & pre releases
 
-$VERSION = "3.004001";      # update after release (same format as perl $])
-#$IS_DEVEL_BUILD = 1;        # change for release versions
 
 # Used during the prerelease/release-candidate part of the official release
 # process. If you hacked up your SA, you should add a version_tag to your .cf
 # files; this variable should not be modified.
-#@EXTRA_VERSION = qw();
+our @EXTRA_VERSION = qw();
 
-@ISA = qw();
+our @ISA = qw();
 
 # SUB_VERSION is now just <yyyy>-<mm>-<dd>
-$SUB_VERSION = 'svnunknown';
-if ('$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $' =~ ':') {
-  # Subversion keyword "$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $" has been successfully expanded.
+our $SUB_VERSION = 'svnunknown';
+if ('$LastChangedDate: 2019-12-06 18:58:14 -0500 (Fri, 06 Dec 2019) $' =~ ':') {
+  # Subversion keyword "$LastChangedDate: 2019-12-06 18:58:14 -0500 (Fri, 06 Dec 2019) $" has been successfully expanded.
   # Doesn't happen with automated launchpad builds:
   # https://bugs.launchpad.net/launchpad/+bug/780916
-  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2015-04-28 03:54:45 -0400 (Tue, 28 Apr 2015) $ updated by SVN'))[1];
+  $SUB_VERSION = (split(/\s+/,'$LastChangedDate: 2019-12-06 18:58:14 -0500 (Fri, 06 Dec 2019) $ updated by SVN'))[1];
 }
 
 
 if (defined $IS_DEVEL_BUILD && $IS_DEVEL_BUILD) {
-  if ('$LastChangedRevision: 1676427 $' =~ ':') {
-    # Subversion keyword "$LastChangedRevision: 1676427 $" has been successfully expanded.
-    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1676427 $ updated by SVN}[1]));
+  if ('$LastChangedRevision: 1870940 $' =~ ':') {
+    # Subversion keyword "$LastChangedRevision: 1870940 $" has been successfully expanded.
+    push(@EXTRA_VERSION, ('r' . qw{$LastChangedRevision: 1870940 $ updated by SVN}[1]));
   } else {
     push(@EXTRA_VERSION, ('r' . 'svnunknown'));
   }
@@ -128,13 +123,14 @@ sub Version {
   return join('-', sprintf("%d.%d.%d", $1, $2, $3), @EXTRA_VERSION);
 }
 
-$HOME_URL = "http://spamassassin.apache.org/";
+our $HOME_URL = "https://spamassassin.apache.org/";
 
 # note that the CWD takes priority.  This is required in case a user
 # is testing a new version of SpamAssassin on a machine with an older
 # version installed.  Unless you can come up with a fix for this that
 # allows "make test" to work, don't change this.
-@default_rules_path = (
+
+our @default_rules_path = (
   '__local_state_dir__/__version__',
   '__def_rules_dir__',
   '__prefix__/share/spamassassin',
@@ -143,7 +139,7 @@ $HOME_URL = "http://spamassassin.apache.org/";
 );
 
 # first 3 are BSDish, latter 2 Linuxish
-@site_rules_path = (
+our @site_rules_path = (
   '__local_rules_dir__',
   '__prefix__/etc/mail/spamassassin',
   '__prefix__/etc/spamassassin',
@@ -154,21 +150,23 @@ $HOME_URL = "http://spamassassin.apache.org/";
   '/etc/spamassassin',
 );
 
-@default_prefs_path = (
+our @default_prefs_path = (
   '__local_rules_dir__/user_prefs.template',
   '__prefix__/etc/mail/spamassassin/user_prefs.template',
   '__prefix__/share/spamassassin/user_prefs.template',
+  '__local_state_dir__/__version__/updates_spamassassin_org/user_prefs.template',
+  '__def_rules_dir__/user_prefs.template',
   '/etc/spamassassin/user_prefs.template',
   '/etc/mail/spamassassin/user_prefs.template',
   '/usr/local/share/spamassassin/user_prefs.template',
   '/usr/share/spamassassin/user_prefs.template',
 );
 
-@default_userprefs_path = (
+our @default_userprefs_path = (
   '~/.spamassassin/user_prefs',
 );
 
-@default_userstate_dir = (
+our @default_userstate_dir = (
   '~/.spamassassin',
 );
 
@@ -429,7 +427,6 @@ sub new {
   }
 
   $self->{conf} ||= new Mail::SpamAssassin::Conf ($self);
-  $self->{registryboundaries} = Mail::SpamAssassin::RegistryBoundaries->new ($self);
   $self->{plugins} = Mail::SpamAssassin::PluginHandler->new ($self);
 
   $self->{save_pattern_hits} ||= 0;
@@ -548,6 +545,8 @@ sub parse {
   my $msg = Mail::SpamAssassin::Message->new({
     message=>$message, parsenow=>$parsenow,
     normalize=>$self->{conf}->{normalize_charset},
+    body_part_scan_size=>$self->{conf}->{body_part_scan_size},
+    rawbody_part_scan_size=>$self->{conf}->{rawbody_part_scan_size},
     master_deadline=>$master_deadline, suppl_attrib=>$suppl_attrib });
 
   # bug 5069: The goal here is to get rendering plugins to do things
@@ -840,7 +839,7 @@ sub signal_user_changed {
   $self->{'learn_to_journal'} = $self->{conf}->{bayes_learn_to_journal};
 
   $set |= 1 unless $self->{local_tests_only};
-  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available();
+  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available() && $self->{conf}->{use_bayes_rules};
 
   $self->{conf}->set_score_set ($set);
 
@@ -1166,7 +1165,7 @@ sub remove_spamassassin_markup {
   $hdrs =~ s/^\n//;
 
 ###########################################################################
-  # Backward Compatibilty, pre 3.0.x.
+  # Backward Compatibility, pre 3.0.x.
 
   # deal with rewritten headers w/out X-Spam-Prev- versions ...
   $self->init(1);
@@ -1273,7 +1272,7 @@ sub read_scoreonly_config {
 
 =item $f->load_scoreonly_sql ($username)
 
-Read configuration paramaters from SQL database and parse scores from it.  This
+Read configuration parameters from SQL database and parse scores from it.  This
 will only take effect if the perl C<DBI> module is installed, and the
 configuration parameters C<user_scores_dsn>, C<user_scores_sql_username>, and
 C<user_scores_sql_password> are set correctly.
@@ -1299,7 +1298,7 @@ sub load_scoreonly_sql {
 
 =item $f->load_scoreonly_ldap ($username)
 
-Read configuration paramaters from an LDAP server and parse scores from it.
+Read configuration parameters from an LDAP server and parse scores from it.
 This will only take effect if the perl C<Net::LDAP> and C<URI> modules are
 installed, and the configuration parameters C<user_scores_dsn>,
 C<user_scores_ldap_username>, and C<user_scores_ldap_password> are set
@@ -1348,7 +1347,7 @@ plan to fork() or start a new perl interpreter thread to process a message,
 this is suboptimal, as each process/thread will have to perform these actions.
 
 Call this function in the master thread or process to perform the actions
-straightaway, so that the sub-processes will not have to.
+straight away, so that the sub-processes will not have to.
 
 If C<$use_user_prefs> is 0, this will initialise the SpamAssassin
 configuration without reading the per-user configuration file and it will
@@ -1777,7 +1776,7 @@ sub init {
   # Figure out/set our initial scoreset
   my $set = 0;
   $set |= 1 unless $self->{local_tests_only};
-  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available();
+  $set |= 2 if $self->{bayes_scanner} && $self->{bayes_scanner}->is_scan_available() && $self->{conf}->{use_bayes_rules};
   $self->{conf}->set_score_set ($set);
 
   if ($self->{only_these_rules}) {
@@ -2120,14 +2119,22 @@ sub have_plugin {
 
 sub call_plugins {
   my $self = shift;
+  my $subname = shift;
 
   # We could potentially get called after a finish(), so just return.
   return unless $self->{plugins};
 
+  # Use some calls ourself too
+  if ($subname eq 'finish_parsing_end') {
+    # Initialize RegistryBoundaries, now that util_rb_tld etc from config is
+    # read.  Plugins can also now use {valid_tlds_re} to one time compile
+    # regexes in finish_parsing_end.
+    $self->{registryboundaries} = Mail::SpamAssassin::RegistryBoundaries->new ($self);
+  }
+
   # safety net in case some plugin changes global settings, Bug 6218
   local $/ = $/;  # prevent underlying modules from changing the global $/
 
-  my $subname = shift;
   return $self->{plugins}->callback($subname, @_);
 }
 
@@ -2278,8 +2285,8 @@ C<Sys::Syslog>
 
 =head1 MORE DOCUMENTATION
 
-See also E<lt>http://spamassassin.apache.org/E<gt> and
-E<lt>http://wiki.apache.org/spamassassin/E<gt> for more information.
+See also E<lt>https://spamassassin.apache.org/E<gt> and
+E<lt>https://wiki.apache.org/spamassassin/E<gt> for more information.
 
 =head1 SEE ALSO
 
@@ -2290,11 +2297,11 @@ sa-update(1)
 
 =head1 BUGS
 
-See E<lt>http://issues.apache.org/SpamAssassin/E<gt>
+See E<lt>https://issues.apache.org/SpamAssassin/E<gt>
 
 =head1 AUTHORS
 
-The SpamAssassin(tm) Project E<lt>http://spamassassin.apache.org/E<gt>
+The SpamAssassin(tm) Project E<lt>https://spamassassin.apache.org/E<gt>
 
 =head1 COPYRIGHT
 
@@ -2306,6 +2313,6 @@ described in the file C<LICENSE> included with the distribution.
 The latest version of this library is likely to be available from CPAN
 as well as:
 
-  E<lt>http://spamassassin.apache.org/E<gt>
+E<lt>https://spamassassin.apache.org/E<gt>
 
 =cut
